/* - GPS full -
 * This example displays the connected Phidget GPS device's details as well as the current
 * position, velocity, heading. There is also a map view that shows the current location on a map.
 *
 * Please note that this example was designed to work with only one Phidget GPS connected.
 * For an example showing how to use two Phidgets of the same time concurrently, please see the
 * Servo-multi example in the Servo Examples.
 *
 * Copyright 2010 Phidgets Inc.
 * This work is licensed under the Creative Commons Attribution 2.5 Canada License.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by/2.5/ca/
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Runtime.InteropServices;
using System.Diagnostics;
using GMap.NET;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace GPS_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
		formCleaner cleaner;
		Graphics mapGraphic;
		private GPS gps;
		private ErrorEventBox errorBox;

		public Form1() {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			open.commandLine = commandLine;
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {

			errorBox = new ErrorEventBox(this);
			gps = new GPS();

			gps.Attach += gps_Attach;
			gps.Detach += gps_Detach;
			gps.Error += gps_Error;

			gps.PositionChange += gps_PositionChange;
			gps.PositionFixStateChange += gps_PositionFixStateChange;
			gps.HeadingChange += Gps_HeadingChange;

			commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters

			try { //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
				gps.Channel = phidgetParameters.Channel; //selects the channel on the device to open
				gps.DeviceSerialNumber = phidgetParameters.SerialNumber; //selects the device or hub to open
				gps.HubPort = phidgetParameters.HubPort; //selects the port on the hub to open
				gps.IsHubPortDevice = phidgetParameters.isHubPortDevice; //is the device a port on a VINT hub?

				if (phidgetParameters.isRemote) //are we trying to open a remote device?
				{
					gps.IsRemote = true;
					Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
					if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
						Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
				} else
					gps.IsLocal = true;

				gps.Open(); //open the device specified by the above parameters
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }

			mapBox.MapProvider = GMap.NET.MapProviders.GoogleMapProvider.Instance;
			mapBox.Position = new GMap.NET.PointLatLng(0, 0);
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			gps.Close();
		}

		void gps_Attach(object sender, AttachEventArgs e) {
			GPS attached = (GPS)sender;
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);
			timer1.Start();

			mapGraphic = pictureBox1.CreateGraphics();

			dataPanel.Visible = true;
			outputBox.Visible = true;
		}

		void gps_Detach(object sender, DetachEventArgs e) {
			dataPanel.Visible = false;
			GPS detached = (GPS)sender;
			phidgetInfoBox1.Clear();
			cleaner.clean();
			checkBox1.Checked = false;
			outputBox.Visible = false;

			timer1.Stop();

			timeTxt.Text = "Unknown";
			dateTxt.Text = "Unknown";

			latTxt.Text = "Unknown";
			longTxt.Text = "Unknown";
			altTxt.Text = "Unknown";
			headTxt.Text = "Unknown";
			velTxt.Text = "Unknown";

			mapBox.Visible = false;
			mapViewButton.Visible = false;
			satelliteViewButton.Visible = false;
			hybridViewButton.Visible = false;
		}

		void gps_Error(object sender, ErrorEventArgs e) {
			errorBox.addMessage(e.Description);
		}

		void gps_PositionChange(object sender, GPSPositionChangeEventArgs e) {
			latTxt.Text = e.Latitude.ToString("f6");
			longTxt.Text = e.Longitude.ToString("f6");
			altTxt.Text = e.Altitude.ToString("f1") + " m";

			try {
				if (checkBox1.Checked)
					mapBox.Position = new GMap.NET.PointLatLng(e.Latitude, e.Longitude);
			} catch { }
		}

		private void Gps_HeadingChange(Object sender, GPSHeadingChangeEventArgs e) {
			headTxt.Text = e.Heading.ToString();
			velTxt.Text = e.Velocity.ToString("f1") + " km/h";

			double xcoor1 = 0, ycoor1 = 0, xcoor2 = 0, ycoor2 = 0;
			double head = e.Heading;
			xcoor1 = (Math.Sin(((head + 180) * 0.0174532925)) * 10) + 10;
			ycoor1 = (Math.Cos(((head + 180) * 0.0174532925)) * 10) + 10;
			xcoor2 = 10 - (Math.Sin(((head + 180) * 0.0174532925)) * 10);
			ycoor2 = 10 - (Math.Cos(((head + 180) * 0.0174532925)) * 10);

			mapGraphic.Clear(Color.FromName("Control"));
			mapGraphic.SmoothingMode = SmoothingMode.AntiAlias;
			Pen p = new Pen(Color.Black, 4);
			p.StartCap = LineCap.Round;
			p.EndCap = LineCap.ArrowAnchor;
			mapGraphic.DrawLine(p, (float)xcoor2, (float)ycoor2, (float)xcoor1, (float)ycoor1);
			p.Dispose();
		}

		void gps_PositionFixStateChange(object sender, GPSPositionFixStateChangeEventArgs e) {
			checkBox1.Checked = e.PositionFixState;
			if (e.PositionFixState == false) {
				latTxt.Text = "Unknown";
				longTxt.Text = "Unknown";
				altTxt.Text = "Unknown";
				headTxt.Text = "Unknown";
				velTxt.Text = "Unknown";
			}
			mapBox.Visible = e.PositionFixState;
			mapViewButton.Visible = e.PositionFixState;
			satelliteViewButton.Visible = e.PositionFixState;
			hybridViewButton.Visible = e.PositionFixState;
		}

		private void timer1_Tick(object sender, EventArgs e) {
			if (gps.Attached) {
				try {
					//GPS time is in UTC, convert to local time
					DateTime localTime = gps.DateAndTime.ToLocalTime();

					timeTxt.Text = localTime.Hour.ToString("D2") + ":" + localTime.Minute.ToString("D2") + ":" + localTime.Second.ToString("D2") + "." + localTime.Millisecond.ToString("D3");
					dateTxt.Text = localTime.Day + "/" + localTime.Month + "/" + localTime.Year;
				} catch { }
			}
		}

		private void mapViewButton_Click(object sender, EventArgs e) {
			mapBox.MapProvider = GMap.NET.MapProviders.GoogleMapProvider.Instance;
		}

		private void hybridViewButton_Click(object sender, EventArgs e) {
			mapBox.MapProvider = GMap.NET.MapProviders.GoogleHybridMapProvider.Instance;
		}

		private void satelliteViewButton_Click(object sender, EventArgs e) {
			mapBox.MapProvider = GMap.NET.MapProviders.GoogleSatelliteMapProvider.Instance;
		}
	}
}